@@ -1,12 +1,12 @@
 use blsttc::{PublicKeySet, SignatureShare};
-use std::collections::{BTreeSet, HashMap, HashSet};
+use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::iter::FromIterator;
 
 use curve25519_dalek_ng::scalar::Scalar;
 
 use crate::{
-    Amount, AmountSecrets, Dbc, DbcContent, Error, NodeSignature, PublicKey, ReissueShare,
-    ReissueTransaction, Result, SpendKey,
+    Amount, AmountSecrets, Dbc, DbcContent, Error, NodeSignature, PublicKey, ReissueRequest,
+    ReissueShare, ReissueTransaction, Result, SpendKey, SpentProof, SpentProofShare,
 };
 
 ///! Unblinded data for creating sn_dbc::DbcContent
@@ -95,12 +95,84 @@ impl TransactionBuilder {
     }
 }
 
+/// Builds a ReissueRequest from a ReissueTransaction and
+/// any number of (input) DBC spent proof shares.
+#[derive(Debug)]
+pub struct ReissueRequestBuilder {
+    pub reissue_transaction: ReissueTransaction,
+    pub spent_proof_shares: BTreeMap<SpendKey, HashSet<SpentProofShare>>,
+}
+
+impl ReissueRequestBuilder {
+    /// Create a new ReissueRequestBuilder from a ReissueTransaction
+    pub fn new(reissue_transaction: ReissueTransaction) -> Self {
+        Self {
+            reissue_transaction,
+            spent_proof_shares: Default::default(),
+        }
+    }
+
+    /// Add a SpentProofShare for the given spend_key
+    pub fn add_spent_proof_share(mut self, spend_key: SpendKey, share: SpentProofShare) -> Self {
+        let shares = self.spent_proof_shares.entry(spend_key).or_default();
+        shares.insert(share);
+
+        self
+    }
+
+    pub fn build(&self) -> Result<ReissueRequest> {
+        let spent_proofs: BTreeMap<SpendKey, SpentProof> = self
+            .spent_proof_shares
+            .iter()
+            .map(|(spend_key, shares)| {
+                let any_share = shares
+                    .iter()
+                    .next()
+                    .ok_or(Error::ReissueRequestMissingSpentProofShare(*spend_key))?;
+
+                if shares
+                    .iter()
+                    .map(SpentProofShare::spentbook_pks)
+                    .any(|pks| pks != any_share.spentbook_pks())
+                {
+                    return Err(Error::ReissueRequestPublicKeySetMismatch);
+                }
+
+                let spent_sig = any_share.spent_sig.clone();
+                let spentbook_pub_key = any_share.spentbook_public_key();
+                let spentbook_sig = any_share.spentbook_pks.combine_signatures(
+                    shares
+                        .iter()
+                        .map(SpentProofShare::spentbook_sig_share)
+                        .map(NodeSignature::threshold_crypto),
+                )?;
+
+                let spent_proof = SpentProof {
+                    spent_sig,
+                    spentbook_pub_key,
+                    spentbook_sig,
+                };
+
+                Ok((*spend_key, spent_proof))
+            })
+            .collect::<Result<_>>()?;
+
+        let transaction = self.reissue_transaction.clone();
+
+        let rr = ReissueRequest {
+            transaction,
+            spent_proofs,
+        };
+        Ok(rr)
+    }
+}
+
 /// A Builder for aggregating ReissueShare (Mint::reissue() results)
 /// from multiple mint nodes and combining signatures to
 /// generate the final Dbc outputs.
-#[derive(Default)]
+#[derive(Debug)]
 pub struct DbcBuilder {
-    pub reissue_transaction: Option<ReissueTransaction>,
+    pub reissue_transaction: ReissueTransaction,
     pub reissue_shares: Vec<ReissueShare>,
 }
 
@@ -108,7 +180,7 @@ impl DbcBuilder {
     /// Create a new DbcBuilder from a ReissueTransaction
     pub fn new(reissue_transaction: ReissueTransaction) -> Self {
         Self {
-            reissue_transaction: Some(reissue_transaction),
+            reissue_transaction,
             reissue_shares: Default::default(),
         }
     }
@@ -119,26 +191,18 @@ impl DbcBuilder {
         self
     }
 
-    /// Set the ReissueTransaction
-    pub fn set_reissue_transaction(mut self, reissue_transaction: ReissueTransaction) -> Self {
-        self.reissue_transaction = Some(reissue_transaction);
+    /// Add multiple ReissueShare from Mint::reissue()
+    pub fn add_reissue_shares(mut self, shares: impl IntoIterator<Item = ReissueShare>) -> Self {
+        self.reissue_shares.extend(shares);
         self
     }
 
     /// Build the output DBCs
-    ///
-    /// Note that the result Vec may be empty if the ReissueTransaction
-    /// has not been set or no ReissueShare has been added.
     pub fn build(self) -> Result<Vec<Dbc>> {
         if self.reissue_shares.is_empty() {
             return Err(Error::NoReissueShares);
         }
 
-        let reissue_transaction = match self.reissue_transaction {
-            Some(rt) => rt,
-            None => return Err(Error::NoReissueTransaction),
-        };
-
         let mut mint_sig_shares: Vec<NodeSignature> = Default::default();
         let mut pk_set: HashSet<PublicKeySet> = Default::default();
 
@@ -161,17 +225,17 @@ impl DbcBuilder {
             pk_set = &pk_set | &pub_key_sets; // union the sets together.
 
             // Verify transaction returned to us by the Mint matches our request
-            if reissue_transaction.blinded() != rs.dbc_transaction {
+            if self.reissue_transaction.blinded() != rs.dbc_transaction {
                 return Err(Error::ReissueShareDbcTransactionMismatch);
             }
 
             // Verify that mint sig count matches input count.
-            if rs.mint_node_signatures.len() != reissue_transaction.inputs.len() {
+            if rs.mint_node_signatures.len() != self.reissue_transaction.inputs.len() {
                 return Err(Error::ReissueShareMintNodeSignaturesLenMismatch);
             }
 
             // Verify that each input has a NodeSignature
-            for input in reissue_transaction.inputs.iter() {
+            for input in self.reissue_transaction.inputs.iter() {
                 if rs.mint_node_signatures.get(&input.spend_key()).is_none() {
                     return Err(Error::ReissueShareMintNodeSignatureNotFoundForInput);
                 }
@@ -201,13 +265,15 @@ impl DbcBuilder {
         let mint_sig = mint_public_key_set.combine_signatures(mint_sig_shares_ref)?;
 
         // Form the final output DBCs, with Mint's Signature for each.
-        let mut output_dbcs: Vec<Dbc> = reissue_transaction
+        let mut output_dbcs: Vec<Dbc> = self
+            .reissue_transaction
             .outputs
             .iter()
             .map(|content| Dbc {
                 content: content.clone(),
                 transaction: dbc_transaction.clone(),
-                transaction_sigs: reissue_transaction
+                transaction_sigs: self
+                    .reissue_transaction
                     .inputs
                     .iter()
                     .map(|input| {
